﻿local GCD = LibStub("AceAddon-3.0"):NewAddon("GCD", "AceConsole-3.0", "AceEvent-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("GCD")
local ACD = LibStub("AceConfigDialog-3.0")

local GetTime = GetTime
local sin, cos, rad, ceil, floor = math.sin, math.cos, math.rad, math.ceil, math.floor
local BOOKTYPE_SPELL = BOOKTYPE_SPELL

local GCDBar, startTime, duration, spellNum, warned, castStartTime, castEndTime, castDuration, castInterrupted, castLatency, castSent, isCasting, isGCD

local cdFrames = {}
local frameBuf = {
	["Frame"] = {},
	["Button"] = {}
}

local cdSpells = {
	["DRUID"] = {
		{['spellID'] = 53251, ['xOff'] = 0, ['yOff'] = 0},
		{['spellID'] = 18562, ['xOff'] = 0, ['yOff'] = 0},
		{['spellID'] = 17116, ['xOff'] = 0, ['yOff'] = 0}
	},
	["PALADIN"] = {
		{['spellID'] = 48825, ['xOff'] = 0, ['yOff'] = 0}
	},
	["PRIEST"] = {
		{['spellID'] = 53007, ['xOff'] = 0, ['yOff'] = 0},
		{['spellID'] = 48113, ['xOff'] = 0, ['yOff'] = 0},
		{['spellID'] = 48089, ['xOff'] = 0, ['yOff'] = 0}
	}
}

local spells = {
	["DRUID"] = 5185,
	["PALADIN"] = 635,
	["PRIEST"] = 2050,
	["SHAMAN"] = 331,
	["WARRIOR"] = 6673,
	["DEATHKNIGHT"] = 45902,
	["HUNTER"] = 75,
	["MAGE"] = 1459,
	["WARLOCK"] = 687,
	["ROGUE"] = 1752,
}

local defaults = {
	char = {
		spark = true,
		spellcds = false,
		casttime = false,
		hidecastbar = false,
	},
	profile = {
		spark = {
			color = {r=1, g=1, b=1, a=1},
			texture = "Spells\\AURA_01"
		},
		ring = {
			color = {r=0, g=1, b=0, a=0.5},
			texture = "Spells\\AURARUNE8",
			rotate = true
		},
		casttime = {
			castcolor = {r=0, g=1, b=0, a=1},
			latencycolor = {r=1, g=0, b=0, a=1},
			interruptcolor = {r=1, g=1, b=0, a=1},
			texture = "Spells\\AURA_01",
		},
	}
}

local createFrame = function(frameType, frameName, frameParent)
	local frame = nil
	if #frameBuf[frameType] > 0 then
		frame = tremove(frameBuf[frameType])
	else
		frame = CreateFrame(frameType, frameName, frameParent)
	end
	return frame
end

local deleteFrame = function(frame)
	frame:Hide()
	tinsert(frameBuf[frame:GetObjectType()], frame)
end

function GCD:InitializeOptions()
	self.options = {
		name = "GCD",
		type = "group",
		args = {
			unlock = {
				name = L["Unlock Icons"],
				type = "execute",
				func = GCD.Unlock,
				order = 100
			},
			info = {
				name = L["All specified colors are vertex colors! (see wowwiki.com)"],
				type = "description",
				order = 200
			},
			spark = {
				name = L["Spark"],
				type = "group",
				args = {
					enabled = {
						name = L["Enabled"],
						type = "toggle",
						get = function(info) return self.db.char.spark end,
						set = function(info,val)
								self.db.char.spark = val
							end,
						order = 100
					},
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.spark.texture end,
						set = function(info, val)
									self.db.profile.spark.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					color = {
						name = L["Color"],
						type = "color",
						get = function(info) return self.db.profile.spark.color.r, self.db.profile.spark.color.g, self.db.profile.spark.color.b, self.db.profile.spark.color.a end,
						set = function(info, r, g, b, a)
									self.db.profile.spark.color = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
					defaults = {
						name = L["Restore defaults"],
						type = "execute",
						func = function()
									self.db.profile.spark = defaults.profile.spark
								end,
						order = 100
					},
				}
			},
			ring = {
				name = L["Ring"],
				type = "group",
				args = {
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.ring.texture end,
						set = function(info, val)
									self.db.profile.ring.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					color = {
						name = L["Color"],
						type = "color",
						get = function(info) return self.db.profile.ring.color.r, self.db.profile.ring.color.g, self.db.profile.ring.color.b, self.db.profile.ring.color.a end,
						set = function(info, r, g, b, a)
									self.db.profile.ring.color = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
					rotate = {
					name = L["Rotate"],
					type = "toggle",
					get = function(info) return self.db.profile.ring.rotate end,
					set = function(info, val) self.db.profile.ring.rotate = val end,
					order = 300
					},
					defaults = {
						name = L["Restore defaults"],
						type = "execute",
						func = function()
									self.db.profile.ring = defaults.profile.ring
								end,
						order = 100
					},
				}
			},
			casttime = {
				name = L["Casttime"],
				type = "group",
				args = {
					enabled = {
						name = L["Enabled"],
						type = "toggle",
						get = function(info) return self.db.char.casttime end,
						set = function(info,val)
								self.db.char.casttime = val
								if val then
									self:RegisterEvent("UNIT_SPELLCAST_SENT")
									self:RegisterEvent("UNIT_SPELLCAST_START")
									self:RegisterEvent("UNIT_SPELLCAST_STOP")
									self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
									self:RegisterEvent("UNIT_SPELLCAST_FAILED")
									self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
									self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
								else
									self:UnregisterEvent("UNIT_SPELLCAST_SENT")
									self:UnregisterEvent("UNIT_SPELLCAST_START")
									self:UnregisterEvent("UNIT_SPELLCAST_STOP")
									self:UnregisterEvent("UNIT_SPELLCAST_INTERRUPTED")
									self:UnregisterEvent("UNIT_SPELLCAST_FAILED")
									self:UnregisterEvent("UNIT_SPELLCAST_DELAYED")
									self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_START")
									self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
									self:UnregisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
								end
							end,
						order = 100
					},
					texture = {
						name = L["Texture"],
						type = "input",
						get = function(info) return self.db.profile.casttime.texture end,
						set = function(info, val)
									self.db.profile.casttime.texture = val
									self:ApplySettings()
								end,
						order = 100
					},
					castcolor = {
						name = L["Cast color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.castcolor.r, self.db.profile.casttime.castcolor.g, self.db.profile.casttime.castcolor.b, self.db.profile.casttime.castcolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.castcolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 200
					},
					latencycolor = {
						name = L["Latency color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.latencycolor.r, self.db.profile.casttime.latencycolor.g, self.db.profile.casttime.latencycolor.b, self.db.profile.casttime.latencycolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.latencycolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 300
					},
					interruptcolor = {
						name = L["Interrupt color"],
						type = "color",
						get = function(info) return self.db.profile.casttime.interruptcolor.r, self.db.profile.casttime.interruptcolor.g, self.db.profile.casttime.interruptcolor.b, self.db.profile.casttime.interruptcolor.a end,
						set = function(info, r, g, b, a)
									self.db.profile.casttime.interruptcolor = {r=r, g=g, b=b, a=a}
									self:ApplySettings()
								end,
						hasAlpha = true,
						order = 300
					},
					hideblizzard = {
						name = L["Hide default castbar"],
						type = "toggle",
						get = function(info) return self.db.char.hidecastbar end,
						set = function(info,val)
								self.db.char.hidecastbar = val
								if val then
									CastingBarFrame:UnregisterAllEvents()
									CastingBarFrame:Hide()
								else
									CastingBarFrame:UnregisterAllEvents()
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
									CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
								end
							end,
						order = 100
					},
					defaults = {
						name = L["Restore defaults"],
						type = "execute",
						func = function()
								self.db.profile.casttime = defaults.profile.casttime	end,
						order = 100
					}
				}
			},
			spellcds = {
				name = L["Spellcds"],
				type = "group",
				args = {
					enabled = {
						name = L["Enabled"],
						type = "toggle",
						get = function(info) return self.db.char.spellcds end,
						set = function(info,val)
								self:ApplySettings()
								self.db.char.spellcds = val
								if val then
									self:ACTIONBAR_UPDATE_COOLDOWN()
								else
									for _,v in ipairs(cdFrames) do
										if v.frame then v.frame:Hide() end
									end
								end
							end,
						order = 100
					},
					defaults = {
						name = L["Restore defaults"],
						type = "execute",
						func = function()
								self.db.char.cdSpells = defaults.char.cdSpells
								GCD:SPELLS_CHANGED()
							end,
						order = 100
					}
				}
			}
		}
	}
end

local function OnUpdate(self, elapsed)
	if not isCasting and not isGCD then
		GCDBar:Hide()
	end
	local x, y = GetCursorPosition(UIParent);
	self:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', (x+self.xOff)/self:GetEffectiveScale(), (y+self.yOff)/self:GetEffectiveScale())
		
	if self.spark then
		if GCD.db.profile.ring.rotate then
			self.texture.timer = self.texture.timer + elapsed;
			if ( self.texture.timer > 0.02 ) then
				self.texture.hAngle = self.texture.hAngle + 0.5;
				local s = sin(rad(self.texture.hAngle));
				local c = cos(rad(self.texture.hAngle));
				self.texture:SetTexCoord(	0.5-s, 0.5+c,
													0.5+c, 0.5+s,
													0.5-c, 0.5-s,
													0.5+s, 0.5-c);
				self.texture.timer = 0;
			end
		end
		if isGCD and GCD.db.char.spark then
			local gcdperc = (GetTime() - self.startTime) / self.duration
			gcdperc = gcdperc < 1 and gcdperc or 1			
			local angle = rad(90-gcdperc*360)
			local x, y = cos(angle), sin(angle)
			self.spark:SetPoint('CENTER', self, 'CENTER',x*25,y*25)
			self.spark:Show()
			if gcdperc >= 1 then
				isGCD = false
				self.spark:Hide()
			end
		else
			self.spark:Hide()
		end
		if isCasting and GCD.db.char.casttime then
			local castperc = (1000 * GetTime() - castStartTime) / castDuration
			castperc = castperc < 1 and castperc or 1
			if castperc > 1-castLatency then
				self.castspark:SetVertexColor(GCD.db.profile.casttime.latencycolor.r, GCD.db.profile.casttime.latencycolor.g, GCD.db.profile.casttime.latencycolor.b, GCD.db.profile.casttime.latencycolor.a) -- 1,0,0,1
			elseif castInterrupted then
				self.castspark:SetVertexColor(GCD.db.profile.casttime.interruptcolor.r, GCD.db.profile.casttime.interruptcolor.g, GCD.db.profile.casttime.interruptcolor.b, GCD.db.profile.casttime.interruptcolor.a) -- 1,1,0,1
			else
				self.castspark:SetVertexColor(GCD.db.profile.casttime.castcolor.r, GCD.db.profile.casttime.castcolor.g, GCD.db.profile.casttime.castcolor.b, GCD.db.profile.casttime.castcolor.a) -- 0,1,0,1
			end
			local angle = rad(90-castperc*360)
			local x, y = cos(angle), sin(angle)
			self.castspark:SetPoint('CENTER', self, 'CENTER',x*30,y*30)
			self.castspark:Show()
			if castperc >= 1 then
				isCasting = false
				self.castspark:Hide()
			end
		else
			self.castspark:Hide()
		end
	elseif self.spell and GCD.db.char.spellcds then
		local perc = (GetTime() - self.startTime) / self.duration
		self.texture:SetVertexColor(1.0,perc,perc)
		local dur = floor((self.duration - (GetTime() - self.startTime))*10)/10
		if dur > 99 then
			dur = '>>'
		elseif dur > 10 then
			dur = ceil(dur)
		end
		self.cdtext:SetText(dur)
		if perc >= 1 then
			self:Hide()
		end
	end
end

local function OnHide(self)
	self:SetScript('OnUpdate', nil)
end

local function OnShow(self)
	self:SetScript('OnUpdate', OnUpdate)
end

local function GetSpellID(spellName)
	local spellNum = nil
	for tab = 1, 4 do
		local _, _, offset, numSpells = GetSpellTabInfo(tab)
		for i = (1+offset), (offset+numSpells) do
			local spell = GetSpellName(i, BOOKTYPE_SPELL)
			if strlower(spell) == strlower(spellName) then
				spellNum = i
				break
			end
		end
	end
	return spellNum;
end

function GCD:OnInitialize()
	local _, class = UnitClass("player")
	defaults.char.cdSpells = cdSpells[class]
	for i,v in ipairs(defaults.char.cdSpells) do
		v.xOff = cos(rad(((i - 1) / #defaults.char.cdSpells ) * 360)) * 36 + 10
		v.yOff = sin(rad(((i - 1) / #defaults.char.cdSpells ) * 360)) * 36 - 10
	end
	
	self.db = LibStub("AceDB-3.0"):New("GCDDB", defaults)
	self:InitializeOptions()
	LibStub("AceConfig-3.0"):RegisterOptionsTable("GCD", self.options)
	self:RegisterChatCommand("gcd",self.OpenConfig)
end

function GCD:ApplySettings()
	if not GCDBar then
		GCDBar = createFrame('Frame')
	end
	GCDBar:Hide()
	GCDBar:ClearAllPoints()
	GCDBar:SetFrameStrata('HIGH')
	GCDBar:SetScript('OnShow', OnShow)
	GCDBar:SetScript('OnHide', OnHide)
	GCDBar:SetWidth(64)
	GCDBar:SetHeight(64)
	GCDBar.xOff = 10
	GCDBar.yOff = -10
	
	if not GCDBar.texture then
		GCDBar.texture = GCDBar:CreateTexture(nil, 'ARTWORK')
		GCDBar.texture.timer = 0;
		GCDBar.texture.hAngle = 0;
	end
	GCDBar.texture:SetTexture(self.db.profile.ring.texture) -- Spells\\AURARUNE8
	GCDBar.texture:SetVertexColor(self.db.profile.ring.color.r,self.db.profile.ring.color.g,self.db.profile.ring.color.b,self.db.profile.ring.color.a) -- 0,1,0,0.5
	GCDBar.texture:SetBlendMode('ADD')
	GCDBar.texture:SetWidth(80)
	GCDBar.texture:SetHeight(80)
	GCDBar.texture:SetPoint('CENTER', GCDBar, 'CENTER')
	local s = sin(rad(GCDBar.texture.hAngle));
	local c = cos(rad(GCDBar.texture.hAngle));
	GCDBar.texture:SetTexCoord(0.5-s, 0.5+c,
										0.5+c, 0.5+s,
										0.5-c, 0.5-s,
										0.5+s, 0.5-c);
	
	if not GCDBar.spark then
		GCDBar.spark = GCDBar:CreateTexture(nil, 'DIALOG')
	end
	GCDBar.spark:SetTexture(self.db.profile.spark.texture) -- Spells\\AURA_01
	GCDBar.spark:SetBlendMode('ADD')
	GCDBar.spark:SetVertexColor(self.db.profile.spark.color.r,self.db.profile.spark.color.g,self.db.profile.spark.color.b,self.db.profile.spark.color.a) -- 1,1,1
	GCDBar.spark:SetWidth(30)
	GCDBar.spark:SetHeight(30)
	
	if not GCDBar.castspark then
		GCDBar.castspark = GCDBar:CreateTexture(nil, 'DIALOG')
	end
	GCDBar.castspark:SetTexture(self.db.profile.casttime.texture) -- Spells\\AURA_01
	GCDBar.castspark:SetVertexColor(self.db.profile.casttime.castcolor.r, self.db.profile.casttime.castcolor.g, self.db.profile.casttime.castcolor.b, self.db.profile.casttime.castcolor.a) -- 0,1,0,1
	GCDBar.castspark:SetBlendMode('ADD')
	GCDBar.castspark:SetWidth(30)
	GCDBar.castspark:SetHeight(30)
	
	for i,v in ipairs(cdFrames) do
		if not v.frame and v.spell then
			local frame = createFrame('Frame')

			frame:ClearAllPoints()
			frame:SetFrameStrata('HIGH')
			frame:SetWidth(15*frame:GetEffectiveScale())
			frame:SetHeight(15*frame:GetEffectiveScale())
			frame:SetScript('OnShow', OnShow)
			frame:SetScript('OnHide', OnHide)
			frame:Hide()
			frame.xOff = v.xOff
			frame.yOff = v.yOff
			
			frame.spell = true
			
			frame.texture = frame.texture or frame:CreateTexture(nil, 'ARTWORK')
			frame.texture:SetTexture(v.icon)
			frame.texture:SetVertexColor(1,0,0)
			frame.texture:SetWidth(15*frame:GetEffectiveScale())
			frame.texture:SetHeight(15*frame:GetEffectiveScale())
			frame.texture:SetPoint('CENTER', frame, 'CENTER')
			
			frame.cdtext = frame.cdtext or frame:CreateFontString()
			frame.cdtext:SetFont("Fonts\\FRIZQT__.TTF", 11, "OUTLINE, MONOCHROME")
			frame.cdtext:SetPoint('CENTER', frame, 'CENTER', 0, 1)
			v.frame = frame
		elseif v.frame then
			v.frame.texture:SetTexture(v.icon)
		end
	end
end

function GCD:OpenConfig()
	ACD:Open("GCD")
end

function GCD:OnEnable()
	if self.db.char.casttime then
		self:RegisterEvent("UNIT_SPELLCAST_SENT")
		self:RegisterEvent("UNIT_SPELLCAST_START")
		self:RegisterEvent("UNIT_SPELLCAST_STOP")
		self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		self:RegisterEvent("UNIT_SPELLCAST_FAILED")
		self:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	end
	if self.db.char.hidecastbar then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:Hide()
	end
	self:RegisterEvent("ACTIONBAR_UPDATE_COOLDOWN")
	self:RegisterEvent("SPELLS_CHANGED")
	
	self:SPELLS_CHANGED()
end

function GCD:OnDisable()
	GCDBar:Hide()
	self:UnregisterAllEvents()
	if self.db.char.hidecastbar then
		CastingBarFrame:UnregisterAllEvents()
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_FAILED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_DELAYED")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP")
		CastingBarFrame:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE")
	end
end

function GCD:ACTIONBAR_UPDATE_COOLDOWN()
	if spellNum then
		local start, dur = GetSpellCooldown(spellNum, BOOKTYPE_SPELL)
		if dur > 0 and dur <= 1.5 and self.db.char.spark then
			isGCD = true
			GCDBar.startTime = start
			GCDBar.duration = dur
			GCDBar:Show()
		end
		if self.db.char.spellcds then
			local _, odur = GetSpellCooldown(spellNum, BOOKTYPE_SPELL)
			for _,v in ipairs(cdFrames) do
				if v.spell then
					local start, dur = GetSpellCooldown(v.spell, BOOKTYPE_SPELL)
					if dur > odur then
						v.frame.startTime = start
						v.frame.duration = dur
						v.frame:Show()
					end
				end
			end
		end
	end
end

function GCD:UNIT_SPELLCAST_SENT(_, unit)
	if unit ~= 'player' then return end
	castSent = GetTime()*1000
end

function GCD:UNIT_SPELLCAST_START(_, unit)
	if unit ~= 'player' then return end
	isCasting = true
	castInterrupted = false
	_, _, _, icon, castStartTime, castEndTime = UnitCastingInfo(unit)
	local sendlag= (castSent and castSent > 0) and GetTime()*1000-castSent or 0
	castDuration=castEndTime-castStartTime
	sendlag = sendlag > castDuration and castDuration or sendlag
	castLatency = sendlag/castDuration
	GCDBar:Show()
end

function GCD:UNIT_SPELLCAST_STOP(_, unit)
	if unit ~= 'player' then return end
  isCasting = false
end

function GCD:UNIT_SPELLCAST_FAILED(event, unit)
	if unit ~= 'player' then return end
	isCasting = false
end

function GCD:UNIT_SPELLCAST_INTERRUPTED(event, unit)
	if unit ~= 'player' then return end
	isCasting = false
end

function GCD:UNIT_SPELLCAST_DELAYED(event, unit)
	if unit ~= 'player' then return end
	castInterrupted = true
	_, _, _, _, castStartTime, castEndTime = UnitCastingInfo(unit)
end

function GCD:UNIT_SPELLCAST_CHANNEL_START(event,unit)
	if unit ~= 'player' then return end
	isCasting = true
	_, _, _, _, castStartTime, castEndTime = UnitChannelInfo(unit)
	local sendlag= (castSent and castSent > 0) and GetTime()*1000-castSent or 0
	castDuration=castEndTime-castStartTime
	sendlag = sendlag > castDuration and castDuration or sendlag
	castLatency = sendlag/castDuration
	GCDBar:Show()
end

function GCD:UNIT_SPELLCAST_CHANNEL_STOP(event,unit)
	if unit ~= 'player' then return end
	isCasting = false
end

function GCD:UNIT_SPELLCAST_CHANNEL_UPDATE(event,unit)
	if unit ~= 'player' then return end
	castInterrupted = true
	_, _, _, _, castStartTime, castEndTime = UnitChannelInfo(unit)
end

function GCD:SPELLS_CHANGED()
	local _, class = UnitClass("player")
	spellNum = GetSpellID(GetSpellInfo(spells[class]))
	
	for _,v in ipairs(cdFrames) do
		deleteFrame(v.frame)
	end
	cdFrames = {}
	for i,v in pairs(self.db.char.cdSpells) do
		local spell, _, icon = GetSpellInfo(v.spellID)
		tinsert(cdFrames, {['spell'] = GetSpellID(spell), ['icon'] = icon, ['xOff'] = v.xOff, ['yOff'] = v.yOff})
	end
	self:ApplySettings()
end

function GCD:Unlock()
	local cursor = createFrame("Frame")
	cursor:SetWidth(20)
	cursor:SetHeight(20)
	cursor:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = true, tileSize = 16, edgeSize = 16, 
                                            insets = { left = 4, right = 4, top = 4, bottom = 4 }})
	cursor:SetBackdropColor(255,0,0,1)
	cursor:SetFrameStrata('HIGH')
	cursor:SetPoint("CENTER", UIParent, "CENTER")
	cursor:SetScript("OnMouseUp", function(self, button)
											if button == "RightButton" then
												deleteFrame(self)
												for i,v in ipairs(cdFrames) do
													GCDBar:Hide()
													GCDBar:SetScript('OnShow', OnShow)
													GCDBar:SetScript('OnHide', OnHide)
													v.frame:Hide()
													v.frame:EnableMouse(false)
													v.frame:SetMovable(false)
													v.frame:SetScript("OnMouseDown", nil)
													v.frame:SetScript("OnMouseUp", nil)
													v.frame:SetScript('OnShow', OnShow)
													v.frame:SetScript('OnHide', OnHide)
													
													v.frame.xOff = v.frame:GetLeft() + v.frame:GetWidth() / 2 - (cursor:GetLeft() + cursor:GetWidth() / 2) + 10
													v.frame.yOff = v.frame:GetBottom() + v.frame:GetHeight() / 2 - (cursor:GetBottom() + cursor:GetHeight() / 2) - 10
													GCD.db.char.cdSpells[i].xOff = v.frame.xOff
													GCD.db.char.cdSpells[i].yOff = v.frame.yOff
												end
											end
										end)
	cursor:SetScript("OnEnter", function(self)
											GameTooltip:SetOwner(self, "ANCHOR_CURSOR")
											GameTooltip:SetText("Right Click")
											GameTooltip:AddLine("to lock the icons and save their positions", 1, 1, 1)
											GameTooltip:Show()
										end)
	cursor:SetScript("OnLeave", function(self)
											GameTooltip:Hide()
										end)
	cursor:EnableMouse(true)	
	cursor:Show()
	
	GCDBar:SetPoint("CENTER", UIParent, "CENTER")
	GCDBar:SetScript('OnShow', nil)
	GCDBar:SetScript('OnHide', nil)
	GCDBar:Show()
	
	for i,v in ipairs(cdFrames) do
		v.frame:SetScript('OnShow', nil)
		v.frame:SetScript('OnHide', nil)
		v.frame:SetPoint('CENTER', UIParent, 'BOTTOMLEFT', (cursor:GetLeft()+v.frame.xOff)/v.frame:GetEffectiveScale(), (cursor:GetTop()+v.frame.yOff)/v.frame:GetEffectiveScale())
		v.frame.texture:SetVertexColor(1.0,1.0,1.0)
		v.frame.cdtext:SetText("")
		v.frame:Show()
		v.frame:SetMovable(true)
		v.frame:EnableMouse(true)
		v.frame:SetScript("OnMouseDown", v.frame.StartMoving)
		v.frame:SetScript("OnMouseUp", v.frame.StopMovingOrSizing)
	end
	
	ACD:Close("GCD")
end